@@ -1,6 +1,6 @@
 package PGXN::API::Router;
 
-use 5.10.0;
+use v5.14;
 use utf8;
 use PGXN::API;
 use Plack::Builder;
@@ -13,7 +13,7 @@ use Plack::Request;
 use Encode;
 use File::Spec::Functions qw(catdir);
 use namespace::autoclean;
-our $VERSION = v0.16.5;
+our $VERSION = v0.20.0;
 
 sub app {
     my ($class, %params) = @_;
@@ -48,7 +48,7 @@ sub app {
             };
         };
 
-        # Sever most stuff as plain files.
+        # Serve most stuff as plain files.
         my $files = Plack::App::File->new(root => $root)->to_app;
         mount '/' => sub {
             my $env = shift;
@@ -226,7 +226,7 @@ David E. Wheeler <david.wheeler@pgexperts.com>
 
 =head1 Copyright and License
 
-Copyright (c) 2011-2013 David E. Wheeler.
+Copyright (c) 2011-2024 David E. Wheeler.
 
 This module is free software; you can redistribute it and/or modify it under
 the L<PostgreSQL License|http://www.opensource.org/licenses/postgresql>.
